/*
    UHU-menu -- output plugin for Gnome (2.12)
    Copyright (C) 2003  Koblinger Egmont <egmont@uhulinux.hu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _GNU_SOURCE
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhu-menu.h"

static void indent (FILE *f, int n)
{
	int i;
	for (i = 0; i < n; i++) fprintf(f, "\t");
}

static void write_item (FILE *f, const struct item *item, const char *categories)
{
	int first = 1;
	int cat = 0;
	struct section *s = item->sections;
	struct key *k;
	struct lang *l;
	char *p;
	while (s != NULL) {
		if (!first) fprintf(f, "\n");
		first = 0;
		fprintf(f, "[%s]\n", s->name);
		k = s->keys;
		while (k != NULL) {
			l = k->langs;
			while (l != NULL) {
				if (l->language->name[0] != '\0') {
					fprintf(f, "%s[%s]=", k->name, l->language->name);
				} else {
					fprintf(f, "%s=", k->name);
				}
				p = l->value;
				while (*p != '\0') {
					switch (*p) {
					  case ' ':
						if (p == l->value || *(p+1) == '\0') fprintf(f, "\\s");
						else fputc(' ', f);
						break;
					  case '\n':
						fprintf(f, "\\n");
						break;
					  case '\t':
						fprintf(f, "\\t");
						break;
					  case '\r':
						fprintf(f, "\\r");
						break;
					  case '\\':
						fprintf(f, "\\\\");
						break;
					  default:
						fputc(*p, f);
					}
					p++;
				}
				if (s == item->sections && categories != NULL && !strcmp(k->name, "Categories")) {
					if (p-1 >= l->value && *(p-1) != ';') fprintf(f, ";");
					fprintf(f, "X-UHU-%s;", categories);
					cat = 1;
				}
				fputc('\n', f);
				l = l->next;
			}
			k = k->next;
		}
		if (s == item->sections && !cat && categories != NULL) {
			fprintf(f, "Categories=X-UHU-%s;\n", categories);
		}
		s = s->next;
	}
}

static int file_exists (const char *filename)
{
	struct stat st;
	return lstat(filename, &st) >= 0;
}

static void dump (const struct item *item, const char *path, const char *path_tmp, FILE *ff, int n)
{
	int i, j, l;
	FILE *f;
	char *p;
	char buf[PATH_MAX];
	char buf2[PATH_MAX];

	sprintf(buf, "%s%s", path_tmp, item->file_name);
	if (!item->issubmenu) {
		p = strrchr(item->file_name, '/');
		if (p == NULL) p = item->file_name; else p++;
		if (p[0] == '\0') p = "UHU";
		l = strlen(p);
		if (l > 8 && !strcmp(p + l - 8, ".desktop")) l -= 8;
		strncpy(buf, p, l);
		buf[l] = '\0';
		sprintf(buf2, "%s/applications/%s.desktop", path_tmp, buf);
		j = 0;
		while (file_exists(buf2)) {
			j++;
			sprintf(buf2, "%s/applications/%s-%d.desktop", path_tmp, buf, j);
		}
		f = fopen(buf2, "w");
		if (f == NULL) {
			message_e(3, "fopen %s", buf2);
		} else {
			sprintf(buf, "UHU%s", item->file_name);
			*strrchr(buf, '/') = '\0';
			write_item(f, item, buf);
			fclose(f);
		}
	} else {
		p = strrchr(item->file_name, '/');
		if (p == NULL) p = item->file_name; else p++;
		if (p[0] == '\0') p = "UHU";
		l = strlen(p);
		strncpy(buf, p, l);
		buf[l] = '\0';
		sprintf(buf2, "%s/desktop-directories/%s.directory", path_tmp, buf);
		j = 0;
		while (file_exists(buf2)) {
			j++;
			sprintf(buf2, "%s/desktop-directories/%s-%d.directory", path_tmp, buf, j);
		}
		f = fopen(buf2, "w");
		if (f == NULL) {
			message_e(3, "fopen %s", buf2);
		} else {
			write_item(f, item, NULL);
			fclose(f);
		}
		indent(ff, n);
		fprintf(ff, "<Menu>\n");
		indent(ff, n+1);
		fprintf(ff, "<Name>");
		p = get_value(item, NULL, "Name", "");
		while (*p) {
			if ((*p >= '0' && *p <= '9') ||
			    (*p >= 'A' && *p <= 'Z') ||
			    (*p >= 'a' && *p <= 'z')) fputc(*p, ff);
			else fputc('_', ff);
			p++;
		}
		fprintf(ff, "</Name>\n");
		if (n == 0) {
			indent(ff, n+1);
			fprintf(ff, "<AppDir>%s/applications</AppDir>\n", path);
			indent(ff, n+1);
			fprintf(ff, "<DirectoryDir>%s/desktop-directories</DirectoryDir>\n", path);
		}
		indent(ff, n+1);
		fprintf(ff, "<Directory>%s</Directory>\n", strrchr(buf2, '/') + 1);
		for (i = 0; i < item->subitems_nr; i++) {
			dump(&item->subitems[i], path, path_tmp, ff, n+1);
		}
		indent(ff, n+1);
		fprintf(ff, "<Include>\n");
		indent(ff, n+2);
		fprintf(ff, "<Category>X-UHU-UHU%s</Category>\n", item->file_name);
		indent(ff, n+1);
		fprintf(ff, "</Include>\n");
		indent(ff, n);
		fprintf(ff, "</Menu>\n");
	}
}

int write_menu (const struct item *item, const char *path)
{
	char path_tmp[PATH_MAX];
	char buf[PATH_MAX];
	FILE *ff;

	sprintf(path_tmp, "%s.tmp", path);
	rm_rf(path_tmp);
	mkdir(path_tmp, 0777);
	sprintf(buf, "%s/applications", path_tmp);
	mkdir(buf, 0777);
	sprintf(buf, "%s/desktop-directories", path_tmp);
	mkdir(buf, 0777);
	sprintf(buf, "%s/UHU.menu", path_tmp);
	ff = fopen(buf, "w");
	if (ff == NULL) {
		message_e(3, "fopen %s", buf);
	} else {
		fprintf(ff, "<!DOCTYPE Menu PUBLIC \"-//freedesktop//DTD Menu 1.0//EN\" \"http://www.freedesktop.org/standards/menu-spec/1.0/menu.dtd\">\n");
		if (item->subitems_nr > 0) dump(item, path, path_tmp, ff, 0);
		fclose(ff);
	}
	movetree(path_tmp, path);
	return 0;
}

